@@ -548,6 +548,9 @@ class Store:
           node holds a step.
         """
 
+        # remove _output special key. This is used only by schema_topology.
+        config = without(config, '_output')
+
         if '*' in config:
             self.apply_subschema_config(config['*'])
             config = without(config, '*')
@@ -624,6 +627,16 @@ class Store:
                 self.updater or 'accumulate',
             )
 
+            # All leaf nodes must have a divider, even though a divider
+            # on a branch node higher in the tree will take precedence.
+            # By default, we use the ``set`` divider, except in the case
+            # of processes, where we use the ``null`` divider by
+            # default.
+            if self.topology:
+                self.divider = self.divider or divider_registry.access('null')
+            else:
+                self.divider = self.divider or divider_registry.access('set')
+
             self.properties = deep_merge(
                 self.properties,
                 config.get('_properties', {}))
@@ -1229,10 +1242,6 @@ class Store:
         mother = divide['mother']
         mother_path = (mother,)
         daughters = divide['daughters']
-        initial_state = self.inner[mother].get_value(
-            condition=lambda child: not
-            (isinstance(child.value, Process)),
-            f=lambda child: copy.deepcopy(child))
         daughter_states = self.inner[mother].divide_value()
 
         here = self.path_for()
@@ -1241,8 +1250,7 @@ class Store:
                 zip(daughters, daughter_states):
             # use initial state as default, merge in divided values
             merged_initial_state = deep_merge(
-                copy.deepcopy(initial_state),
-                daughter_state)
+                daughter_state, daughter.get('initial_state', {}))
 
             daughter_key = daughter['key']
             daughter_path = (daughter_key,)
@@ -1579,7 +1587,7 @@ class Store:
 
         if self.leaf:
             state = self
-        else:
+        elif not schema.get('_output'):
             for key, subschema in schema.items():
                 path = topology.get(key)
                 if key == '*':
